/**
 ********************************************************************************
 * @file    ads131m0x.c
 * @author  A. Guzman
 * @date    July 2, 2025
 * @brief   Register I/O implementation for ADS131M0x
 ********************************************************************************
 */

#include "ads131m0x.h"
#include "ads131_reg_defs.h"
#include "SPI_IO_nRF.h"


ADS131M0x_StatusTypeDef ADS131M0x_ReadRegister(void *handle, uint8_t address, uint16_t *value) {
    const struct spi_dt_spec *spi = (const struct spi_dt_spec *)handle;

    uint8_t tx_buf1[3] = {
        (uint8_t)((OPCODE_RREG | ((address & 0x1F) << 7)) >> 8),
        (uint8_t)(OPCODE_RREG | ((address & 0x1F) << 7)),
        0x00
    };
    uint8_t rx_buf1[3] = {0};

    if (SPI_IO_Transfer(spi, tx_buf1, rx_buf1, 3) != 0)
        return ADS131M0x_ERROR;

    uint8_t tx_buf2[3] = {0x00, 0x00, 0x00};
    uint8_t rx_buf2[3] = {0};

    if (SPI_IO_Transfer(spi, tx_buf2, rx_buf2, 3) != 0)
        return ADS131M0x_ERROR;

    *value = ((uint16_t)rx_buf2[0] << 8) | rx_buf2[1];
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131M0x_WriteRegister(void *handle, uint8_t address, uint16_t reg_value) {
    const struct spi_dt_spec *spi = (const struct spi_dt_spec *)handle;

    uint8_t tx_buf[6] = {
        (uint8_t)((OPCODE_WREG | ((address & 0x1F) << 7)) >> 8),
        (uint8_t)(OPCODE_WREG | ((address & 0x1F) << 7)),
        0x00,
        (uint8_t)(reg_value >> 8),
        (uint8_t)(reg_value & 0xFF),
        0x00
    };

    uint8_t rx_buf[6] = {0};

    if (SPI_IO_Transfer(spi, tx_buf, rx_buf, 6) != 0)
        return ADS131M0x_ERROR;

    return ADS131M0x_OK;
}


//****ID Register Getter ****//
ADS131M0x_StatusTypeDef ADS131_GetID(void *handle, uint16_t *id)
{
    if (!id) return ADS131M0x_ERROR;

    if (ADS131M0x_ReadRegister(handle, ADS131M0x_ID, id) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    return ADS131M0x_OK;
}


//****STATUS Register Getters ****//
ADS131M0x_StatusTypeDef ADS131_Status_GetLock(void *handle, ADS131M0x_STATUS_LOCK_t *mode) {
    if (!mode) return ADS131M0x_ERROR;

    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_STATUS, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    *mode = (ADS131M0x_STATUS_LOCK_t)(reg & ~ADS131M0x_STATUS_LOCK_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Status_GetFrameResync(void *handle, ADS131M0x_STATUS_F_RESYNC_t *mode) {
    if (!mode) return ADS131M0x_ERROR;

    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_STATUS, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    *mode = (ADS131M0x_STATUS_F_RESYNC_t)(reg & ~ADS131M0x_STATUS_F_RESYNC_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Status_GetRegMap(void *handle, ADS131M0x_STATUS_REG_MAP_t *mode) {
    if (!mode) return ADS131M0x_ERROR;

    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_STATUS, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    *mode = (ADS131M0x_STATUS_REG_MAP_t)(reg & ~ADS131M0x_STATUS_REG_MAP_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Status_GetCrcError(void *handle, ADS131M0x_STATUS_CRC_ERR_t *mode) {
    if (!mode) return ADS131M0x_ERROR;

    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_STATUS, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    *mode = (ADS131M0x_STATUS_CRC_ERR_t)(reg & ~ADS131M0x_STATUS_CRC_ERR_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Status_GetCrcType(void *handle, ADS131M0x_STATUS_CRC_TYPE_t *mode) {
    if (!mode) return ADS131M0x_ERROR;

    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_STATUS, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    *mode = (ADS131M0x_STATUS_CRC_TYPE_t)(reg & ~ADS131M0x_STATUS_CRC_TYPE_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Status_GetReset(void *handle, ADS131M0x_STATUS_RESET_t *mode) {
    if (!mode) return ADS131M0x_ERROR;

    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_STATUS, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    *mode = (ADS131M0x_STATUS_RESET_t)(reg & ~ADS131M0x_STATUS_RESET_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Status_GetWordLength(void *handle, ADS131M0x_STATUS_WLENGTH_t *mode) {
    if (!mode) return ADS131M0x_ERROR;

    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_STATUS, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    *mode = (ADS131M0x_STATUS_WLENGTH_t)(reg & ~ADS131M0x_STATUS_WLENGTH_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Status_GetDrdy3(void *handle, ADS131M0x_STATUS_DRDYx_t *mode) {
    if (!mode) return ADS131M0x_ERROR;

    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_STATUS, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

        
    *mode = (reg >> 3) & 0x01 ? ADS131M0x_STATUS_DRDYx_NEW_DATA : ADS131M0x_STATUS_DRDYx_NO_NEW_DATA;
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Status_GetDrdy2(void *handle, ADS131M0x_STATUS_DRDYx_t *mode) {
    if (!mode) return ADS131M0x_ERROR;

    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_STATUS, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    *mode = (reg >> 2) & 0x01 ? ADS131M0x_STATUS_DRDYx_NEW_DATA : ADS131M0x_STATUS_DRDYx_NO_NEW_DATA;
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Status_GetDrdy1(void *handle, ADS131M0x_STATUS_DRDYx_t *mode) {
    if (!mode) return ADS131M0x_ERROR;

    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_STATUS, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    *mode = (reg >> 1) & 0x01 ? ADS131M0x_STATUS_DRDYx_NEW_DATA : ADS131M0x_STATUS_DRDYx_NO_NEW_DATA;
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Status_GetDrdy0(void *handle, ADS131M0x_STATUS_DRDYx_t *mode) {
    if (!mode) return ADS131M0x_ERROR;

    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_STATUS, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    *mode = reg & 0x01 ? ADS131M0x_STATUS_DRDYx_NEW_DATA : ADS131M0x_STATUS_DRDYx_NO_NEW_DATA;
    return ADS131M0x_OK;
}

//****MODE Register Getters ****//

ADS131M0x_StatusTypeDef ADS131_Mode_GetRegCrcEn(void *handle, ADS131M0x_MODE_REG_CRC_EN_t* mode){
    if (!mode) return ADS131M0x_ERROR;

    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_MODE, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    *mode = (ADS131M0x_MODE_REG_CRC_EN_t)(reg & ~ADS131M0x_MODE_REG_CRC_EN_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Mode_SetRegCrcEn(void *handle, ADS131M0x_MODE_REG_CRC_EN_t mode){
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_MODE, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    reg &= ADS131M0x_MODE_REG_CRC_EN_MASK;
    reg |= mode;

    return ADS131M0x_WriteRegister(handle, ADS131M0x_MODE, reg);
}

ADS131M0x_StatusTypeDef ADS131_Mode_GetRXCrcEn(void *handle, ADS131M0x_MODE_RX_CRC_EN_t* mode){
    if (!mode) return ADS131M0x_ERROR;

    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_MODE, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    *mode = (ADS131M0x_MODE_RX_CRC_EN_t)(reg & ~ADS131M0x_MODE_RX_CRC_EN_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Mode_SetRXCrcEn(void *handle, ADS131M0x_MODE_RX_CRC_EN_t mode){
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_MODE, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    reg &= ADS131M0x_MODE_RX_CRC_EN_MASK;
    reg |= mode;

    return ADS131M0x_WriteRegister(handle, ADS131M0x_MODE, reg);
}

ADS131M0x_StatusTypeDef ADS131_Mode_GetCrcType(void *handle, ADS131M0x_MODE_CRC_TYPE_t* mode){
    if (!mode) return ADS131M0x_ERROR;

    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_MODE, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    *mode = (ADS131M0x_MODE_CRC_TYPE_t)(reg & ~ADS131M0x_MODE_CRC_TYPE_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Mode_SetCrcType(void *handle, ADS131M0x_MODE_CRC_TYPE_t mode){
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_MODE, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    reg &= ADS131M0x_MODE_CRC_TYPE_MASK;
    reg |= mode;

    return ADS131M0x_WriteRegister(handle, ADS131M0x_MODE, reg);
}

ADS131M0x_StatusTypeDef ADS131_Mode_GetReset(void *handle, ADS131M0x_MODE_RESET_t* mode){
    if (!mode) return ADS131M0x_ERROR;

    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_MODE, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    *mode = (ADS131M0x_MODE_RESET_t)(reg & ~ADS131M0x_MODE_RESET_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Mode_SetReset(void *handle, ADS131M0x_MODE_RESET_t mode){
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_MODE, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    reg &= ADS131M0x_MODE_RESET_MASK;
    reg |= mode;

    return ADS131M0x_WriteRegister(handle, ADS131M0x_MODE, reg);
}

ADS131M0x_StatusTypeDef ADS131_Mode_GetWordLength(void *handle, ADS131M0x_MODE_WLENGTH_t* mode){
    if (!mode) return ADS131M0x_ERROR;

    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_MODE, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    *mode = (ADS131M0x_MODE_WLENGTH_t)(reg & ~ADS131M0x_MODE_WLENGTH_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Mode_SetWordLength(void *handle, ADS131M0x_MODE_WLENGTH_t mode){
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_MODE, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    reg &= ADS131M0x_MODE_WLENGTH_MASK;
    reg |= mode;

    return ADS131M0x_WriteRegister(handle, ADS131M0x_MODE, reg);
}

ADS131M0x_StatusTypeDef ADS131_Mode_GetTimeout(void *handle, ADS131M0x_MODE_TIMEOUT_t* mode){
    if (!mode) return ADS131M0x_ERROR;

    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_MODE, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    *mode = (ADS131M0x_MODE_TIMEOUT_t)(reg & ~ADS131M0x_MODE_TIMEOUT_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Mode_SetTimeout(void *handle, ADS131M0x_MODE_TIMEOUT_t mode){
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_MODE, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    reg &= ADS131M0x_MODE_TIMEOUT_MASK;
    reg |= mode;

    return ADS131M0x_WriteRegister(handle, ADS131M0x_MODE, reg);
}

ADS131M0x_StatusTypeDef ADS131_Mode_GetDrdySelect(void *handle, ADS131M0x_MODE_DRDY_SEL_t* mode){
    if (!mode) return ADS131M0x_ERROR;

    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_MODE, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    *mode = (ADS131M0x_MODE_DRDY_SEL_t)(reg & ~ADS131M0x_MODE_DRDY_SEL_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Mode_SetDrdySelect(void *handle, ADS131M0x_MODE_DRDY_SEL_t mode){
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_MODE, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    reg &= ADS131M0x_MODE_DRDY_SEL_MASK;
    reg |= mode;

    return ADS131M0x_WriteRegister(handle, ADS131M0x_MODE, reg);
}

ADS131M0x_StatusTypeDef ADS131_Mode_GetDrdyHiZ(void *handle, ADS131M0x_MODE_DRDY_HiZ_t* mode){
    if (!mode) return ADS131M0x_ERROR;

    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_MODE, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    *mode = (ADS131M0x_MODE_DRDY_HiZ_t)(reg & ~ADS131M0x_MODE_DRDY_HiZ_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Mode_SetDrdyHiZ(void *handle, ADS131M0x_MODE_DRDY_HiZ_t mode){
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_MODE, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    reg &= ADS131M0x_MODE_DRDY_HiZ_MASK;
    reg |= mode;

    return ADS131M0x_WriteRegister(handle, ADS131M0x_MODE, reg);
}

ADS131M0x_StatusTypeDef ADS131_Mode_GetDrdyFMT(void *handle, ADS131M0x_MODE_DRDY_FMT_t* mode){
    if (!mode) return ADS131M0x_ERROR;

    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_MODE, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    *mode = (ADS131M0x_MODE_DRDY_FMT_t)(reg & ~ADS131M0x_MODE_DRDY_FMT_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Mode_SetDrdyFMT(void *handle, ADS131M0x_MODE_DRDY_FMT_t mode){
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_MODE, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    reg &= ADS131M0x_MODE_DRDY_FMT_MASK;
    reg |= mode;

    return ADS131M0x_WriteRegister(handle, ADS131M0x_MODE, reg);
}


//****CLOCK Register Getters & Setters ****//

ADS131M0x_StatusTypeDef ADS131_Clock_GetCH3Enable(void *handle, ADS131M0x_CLOCK_CH3_EN_t *mode) {
    if (!mode) return ADS131M0x_ERROR;
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CLOCK, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    *mode = (ADS131M0x_CLOCK_CH3_EN_t)(reg & ~ADS131M0x_CLOCK_CH3_EN_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Clock_SetCH3Enable(void *handle, ADS131M0x_CLOCK_CH3_EN_t mode) {
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CLOCK, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    reg &= ADS131M0x_CLOCK_CH3_EN_MASK;
    reg |= mode;
    return ADS131M0x_WriteRegister(handle, ADS131M0x_CLOCK, reg);
}

ADS131M0x_StatusTypeDef ADS131_Clock_GetCH2Enable(void *handle, ADS131M0x_CLOCK_CH2_EN_t *mode) {
    if (!mode) return ADS131M0x_ERROR;
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CLOCK, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    *mode = (ADS131M0x_CLOCK_CH2_EN_t)(reg & ~ADS131M0x_CLOCK_CH2_EN_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Clock_SetCH2Enable(void *handle, ADS131M0x_CLOCK_CH2_EN_t mode) {
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CLOCK, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    reg &= ADS131M0x_CLOCK_CH2_EN_MASK;
    reg |= mode;
    return ADS131M0x_WriteRegister(handle, ADS131M0x_CLOCK, reg);
}


ADS131M0x_StatusTypeDef ADS131_Clock_GetCH1Enable(void *handle, ADS131M0x_CLOCK_CH1_EN_t *mode) {
    if (!mode) return ADS131M0x_ERROR;
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CLOCK, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    *mode = (ADS131M0x_CLOCK_CH1_EN_t)(reg & ~ADS131M0x_CLOCK_CH1_EN_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Clock_SetCH1Enable(void *handle, ADS131M0x_CLOCK_CH1_EN_t mode) {
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CLOCK, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    reg &= ADS131M0x_CLOCK_CH1_EN_MASK;
    reg |= mode;
    return ADS131M0x_WriteRegister(handle, ADS131M0x_CLOCK, reg);
}

ADS131M0x_StatusTypeDef ADS131_Clock_GetCH0Enable(void *handle, ADS131M0x_CLOCK_CH0_EN_t *mode) {
    if (!mode) return ADS131M0x_ERROR;
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CLOCK, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    *mode = (ADS131M0x_CLOCK_CH0_EN_t)(reg & ~ADS131M0x_CLOCK_CH0_EN_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Clock_SetCH0Enable(void *handle, ADS131M0x_CLOCK_CH0_EN_t mode) {
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CLOCK, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    reg &= ADS131M0x_CLOCK_CH0_EN_MASK;
    reg |= mode;
    return ADS131M0x_WriteRegister(handle, ADS131M0x_CLOCK, reg);
}

ADS131M0x_StatusTypeDef ADS131_Clock_GetTBM(void *handle, ADS131M0x_CLOCK_TBM_t *mode) {
    if (!mode) return ADS131M0x_ERROR;
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CLOCK, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    *mode = (ADS131M0x_CLOCK_TBM_t)(reg & ~ADS131M0x_CLOCK_TBM_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Clock_SetTBM(void *handle, ADS131M0x_CLOCK_TBM_t mode) {
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CLOCK, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    reg &= ADS131M0x_CLOCK_TBM_MASK;
    reg |= mode;
    return ADS131M0x_WriteRegister(handle, ADS131M0x_CLOCK, reg);
}

ADS131M0x_StatusTypeDef ADS131_Clock_GetOSR(void *handle, ADS131M0x_CLOCK_OSR_t *osr) {
    if (!osr) return ADS131M0x_ERROR;
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CLOCK, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    *osr = (ADS131M0x_CLOCK_OSR_t)(reg & ~ADS131M0x_CLOCK_OSR_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Clock_SetOSR(void *handle, ADS131M0x_CLOCK_OSR_t osr) {
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CLOCK, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    reg &= ADS131M0x_CLOCK_OSR_MASK;
    reg |= osr;
    return ADS131M0x_WriteRegister(handle, ADS131M0x_CLOCK, reg);
}

ADS131M0x_StatusTypeDef ADS131_Clock_GetPowerMode(void *handle, ADS131M0x_CLOCK_PWR_t *mode) {
    if (!mode) return ADS131M0x_ERROR;
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CLOCK, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    *mode = (ADS131M0x_CLOCK_PWR_t)(reg & ~ADS131M0x_CLOCK_PWR_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Clock_SetPowerMode(void *handle, ADS131M0x_CLOCK_PWR_t mode) {
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CLOCK, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    reg &= ADS131M0x_CLOCK_PWR_MASK;
    reg |= mode;
    return ADS131M0x_WriteRegister(handle, ADS131M0x_CLOCK, reg);
}

//****GAIN Register Getter & Setter ****//
ADS131M0x_StatusTypeDef ADS131_Gain_SetPGAGain(void *handle, uint8_t channel, uint16_t gain)
{
    if (gain > 7 || channel > 3) return ADS131M0x_ERROR;
    return ADS131M0x_Set_GAIN_PGAGAINx(handle, channel, (ADS131M0x_GAIN_PGAGAIN_t)gain);
}

ADS131M0x_StatusTypeDef ADS131_Gain_GetPGAGain(void *handle, uint8_t channel, uint16_t *gain)
{
    if (!gain || channel > 3) return ADS131M0x_ERROR;

    ADS131M0x_GAIN_PGAGAIN_t enumGain;
    ADS131M0x_StatusTypeDef status = ADS131M0x_Get_GAIN_PGAGAINx(handle, channel, &enumGain);
    if (status != ADS131M0x_OK) return status;

    *gain = (uint16_t)enumGain;
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131M0x_Set_GAIN_PGAGAINx(void *handle, uint8_t channel, ADS131M0x_GAIN_PGAGAIN_t stat)
{
    if (channel > 3 || stat > 7) return ADS131M0x_ERROR;

    uint16_t clear_mask = ~(0x0007 << (channel * 4));
    uint16_t value = stat << (channel * 4);

    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_GAIN, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    reg &= clear_mask;
    reg |= value;

    return ADS131M0x_WriteRegister(handle, ADS131M0x_GAIN, reg);
}

ADS131M0x_StatusTypeDef ADS131M0x_Get_GAIN_PGAGAINx(void *handle, uint8_t channel, ADS131M0x_GAIN_PGAGAIN_t *stat)
{
    if (!stat || channel > 3) return ADS131M0x_ERROR;

    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_GAIN, &reg) != ADS131M0x_OK)
        return ADS131M0x_ERROR;

    *stat = (ADS131M0x_GAIN_PGAGAIN_t)((reg >> (channel * 4)) & 0x7);
    return ADS131M0x_OK;
}


//**** CFG Register Getters & Setters ****//
ADS131M0x_StatusTypeDef ADS131_CFG_GetGC_DLY(void *handle, ADS131M0x_CFG_GC_DLY_t *mode) {
    if (!mode) return ADS131M0x_ERROR;
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CFG, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    *mode = (ADS131M0x_CFG_GC_DLY_t)(reg & ~ADS131M0x_CFG_GC_DLY_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_CFG_SetGC_DLY(void *handle, ADS131M0x_CFG_GC_DLY_t mode) {
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CFG, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    reg &= ADS131M0x_CFG_GC_DLY_MASK;
    reg |= mode;
    return ADS131M0x_WriteRegister(handle, ADS131M0x_CFG, reg);
}

ADS131M0x_StatusTypeDef ADS131_CFG_GetGC_EN(void *handle, ADS131M0x_CFG_GC_EN_t *mode) {
    if (!mode) return ADS131M0x_ERROR;
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CFG, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    *mode = (ADS131M0x_CFG_GC_EN_t)(reg & ~ADS131M0x_CFG_GC_EN_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_CFG_SetGC_EN(void *handle, ADS131M0x_CFG_GC_EN_t mode) {
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CFG, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    reg &= ADS131M0x_CFG_GC_EN_MASK;
    reg |= mode;
    return ADS131M0x_WriteRegister(handle, ADS131M0x_CFG, reg);
}

ADS131M0x_StatusTypeDef ADS131_CFG_GetCD_ALLCH(void *handle, ADS131M0x_CFG_CD_ALLCH_t *mode) {
    if (!mode) return ADS131M0x_ERROR;
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CFG, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    *mode = (ADS131M0x_CFG_CD_ALLCH_t)(reg & ~ADS131M0x_CFG_CD_ALLCH_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_CFG_SetCD_ALLCH(void *handle, ADS131M0x_CFG_CD_ALLCH_t mode) {
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CFG, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    reg &= ADS131M0x_CFG_CD_ALLCH_MASK;
    reg |= mode;
    return ADS131M0x_WriteRegister(handle, ADS131M0x_CFG, reg);
}

ADS131M0x_StatusTypeDef ADS131_CFG_GetCD_NUM(void *handle, ADS131M0x_CFG_CD_NUM_t *mode) {
    if (!mode) return ADS131M0x_ERROR;
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CFG, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    *mode = (ADS131M0x_CFG_CD_NUM_t)(reg & ~ADS131M0x_CFG_CD_NUM_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_CFG_SetCD_NUM(void *handle, ADS131M0x_CFG_CD_NUM_t mode) {
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CFG, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    reg &= ADS131M0x_CFG_CD_NUM_MASK;
    reg |= mode;
    return ADS131M0x_WriteRegister(handle, ADS131M0x_CFG, reg);
}

ADS131M0x_StatusTypeDef ADS131_CFG_GetCD_LEN(void *handle, ADS131M0x_CFG_CD_LEN_t *mode) {
    if (!mode) return ADS131M0x_ERROR;
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CFG, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    *mode = (ADS131M0x_CFG_CD_LEN_t)(reg & ~ADS131M0x_CFG_CD_LEN_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_CFG_SetCD_LEN(void *handle, ADS131M0x_CFG_CD_LEN_t mode) {
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CFG, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    reg &= ADS131M0x_CFG_CD_LEN_MASK;
    reg |= mode;
    return ADS131M0x_WriteRegister(handle, ADS131M0x_CFG, reg);
}

ADS131M0x_StatusTypeDef ADS131_CFG_GetCD_EN(void *handle, ADS131M0x_CFG_CD_EN_t *mode) {
    if (!mode) return ADS131M0x_ERROR;
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CFG, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    *mode = (ADS131M0x_CFG_CD_EN_t)(reg & ~ADS131M0x_CFG_CD_EN_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_CFG_SetCD_EN(void *handle, ADS131M0x_CFG_CD_EN_t mode) {
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CFG, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    reg &= ADS131M0x_CFG_CD_EN_MASK;
    reg |= mode;
    return ADS131M0x_WriteRegister(handle, ADS131M0x_CFG, reg);
}

//*************** THRESHOLD_MSB ***************//
ADS131M0x_StatusTypeDef ADS131_Threshold_GetCD_TH_MSB(void *handle, ADS131M0x_THRSHLD_MSB_CD_TH_MSB_t *mode) {
    if (!mode) return ADS131M0x_ERROR;
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_THRSHLD_MSB, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    *mode = (ADS131M0x_THRSHLD_MSB_CD_TH_MSB_t)(reg & ~ADS131M0x_THRSHLD_MSB_CD_TH_MSB_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Threshold_SetCD_TH_MSB(void *handle, ADS131M0x_THRSHLD_MSB_CD_TH_MSB_t mode) {
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_THRSHLD_MSB, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    reg &= ADS131M0x_THRSHLD_MSB_CD_TH_MSB_MASK;
    reg |= mode;
    return ADS131M0x_WriteRegister(handle, ADS131M0x_THRSHLD_MSB, reg);
}

//*************** THRESHOLD_LSB ***************//
ADS131M0x_StatusTypeDef ADS131_Threshold_GetCD_TH_LSB(void *handle, ADS131M0x_THRSHLD_LSB_CD_TH_LSB_t *mode) {
    if (!mode) return ADS131M0x_ERROR;
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_THRSHLD_LSB, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    *mode = (ADS131M0x_THRSHLD_LSB_CD_TH_LSB_t)(reg & ~ADS131M0x_THRSHLD_LSB_CD_TH_LSB_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Threshold_SetCD_TH_LSB(void *handle, ADS131M0x_THRSHLD_LSB_CD_TH_LSB_t mode) {
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_THRSHLD_LSB, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    reg &= ADS131M0x_THRSHLD_LSB_CD_TH_LSB_MASK;
    reg |= mode;
    return ADS131M0x_WriteRegister(handle, ADS131M0x_THRSHLD_LSB, reg);
}

ADS131M0x_StatusTypeDef ADS131_Threshold_GetDCBLOCK(void *handle, ADS131M0x_THRSHLD_LSB_DCBLOCK_t *mode) {
    if (!mode) return ADS131M0x_ERROR;
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_THRSHLD_LSB, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    *mode = (ADS131M0x_THRSHLD_LSB_DCBLOCK_t)(reg & ~ADS131M0x_THRSHLD_LSB_DCBLOCK_MASK);
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Threshold_SetDCBLOCK(void *handle, ADS131M0x_THRSHLD_LSB_DCBLOCK_t mode) {
    uint16_t reg;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_THRSHLD_LSB, &reg) != ADS131M0x_OK) return ADS131M0x_ERROR;
    reg &= ADS131M0x_THRSHLD_LSB_DCBLOCK_MASK;
    reg |= mode;
    return ADS131M0x_WriteRegister(handle, ADS131M0x_THRSHLD_LSB, reg);
}

//*************** CHx_OCAL / GCAL ***************//
ADS131M0x_StatusTypeDef ADS131_Channel_GetOCAL(void *handle, uint8_t ch, uint32_t *ocal) {
    if (!ocal || ch > 3) return ADS131M0x_ERROR;
    uint16_t msb, lsb;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CH0_OCAL_MSB + (ch * 4), &msb) != ADS131M0x_OK) return ADS131M0x_ERROR;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CH0_OCAL_LSB + (ch * 4), &lsb) != ADS131M0x_OK) return ADS131M0x_ERROR;
    *ocal = ((uint32_t)msb << 16) | lsb;
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Channel_SetOCAL(void *handle, uint8_t ch, uint32_t ocal) {
    if (ch > 3) return ADS131M0x_ERROR;
    uint16_t msb = (uint16_t)(ocal >> 16);
    uint16_t lsb = (uint16_t)(ocal & 0xFFFF);
    if (ADS131M0x_WriteRegister(handle, ADS131M0x_CH0_OCAL_MSB + (ch * 4), msb) != ADS131M0x_OK) return ADS131M0x_ERROR;
    if (ADS131M0x_WriteRegister(handle, ADS131M0x_CH0_OCAL_LSB + (ch * 4), lsb) != ADS131M0x_OK) return ADS131M0x_ERROR;
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Channel_GetGCAL(void *handle, uint8_t ch, uint32_t *gcal) {
    if (!gcal || ch > 3) return ADS131M0x_ERROR;
    uint16_t msb, lsb;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CH0_GCAL_MSB + (ch * 4), &msb) != ADS131M0x_OK) return ADS131M0x_ERROR;
    if (ADS131M0x_ReadRegister(handle, ADS131M0x_CH0_GCAL_LSB + (ch * 4), &lsb) != ADS131M0x_OK) return ADS131M0x_ERROR;
    *gcal = ((uint32_t)msb << 16) | lsb;
    return ADS131M0x_OK;
}

ADS131M0x_StatusTypeDef ADS131_Channel_SetGCAL(void *handle, uint8_t ch, uint32_t gcal) {
    if (ch > 3) return ADS131M0x_ERROR;
    uint16_t msb = (uint16_t)(gcal >> 16);
    uint16_t lsb = (uint16_t)(gcal & 0xFFFF);
    if (ADS131M0x_WriteRegister(handle, ADS131M0x_CH0_GCAL_MSB + (ch * 4), msb) != ADS131M0x_OK) return ADS131M0x_ERROR;
    if (ADS131M0x_WriteRegister(handle, ADS131M0x_CH0_GCAL_LSB + (ch * 4), lsb) != ADS131M0x_OK) return ADS131M0x_ERROR;
    return ADS131M0x_OK;
}
